/*----------------------

   GATE - Geant4 Application for Tomographic Emission 
   OpenGATE Collaboration 
     
   Giovanni Santin <giovanni.santin@cern.ch> 
   Daniel Strul <daniel.strul@iphe.unil.ch> 
     
   Copyright (C) 2002 UNIL/IPHE, CH-1015 Lausanne 

This software is distributed under the terms 
of the GNU Lesser General  Public Licence (LGPL) 
See GATE/LICENSE.txt for further details 
----------------------*/

/*! \file ./includes/GateAnalysis.hh

    $Log: GateAnalysis.hh,v $
    Revision 1.1.1.1  2005/02/03 13:09:53  jdbeen
    gate cluster code for G4.6.2

    Revision 1.8  2002/12/12 19:03:35  gsantin
    verbose setting forward to the trajectory navigator

    Revision 1.7  2002/11/23 22:51:27  gsantin
    RecordVoxels in the output modules, Root 3D Histo with the voxel densities

    Revision 1.6  2002/10/19 14:52:10  dstrul
    Adaptations for Digigate: the output modules now have two virtual methods RecordBeginOfAcquisition() and RecordEndOfAcquisition()

    Revision 1.5  2002/10/19 10:50:11  dstrul
    Adaptations for DigiGate: the retrieving of the hits and digis collections is now handled by methods provided by the base-class GateVOutputModule

    Revision 1.4  2002/08/11 12:08:21  dstrul
    Cosmetic cleanup of doxygen documentation

    Revision 1.3  2002/05/13 12:51:01  gsantin
    documentation added

        
    - GateAnalysis  - by Giovanni.Santin@cern.ch (Apr 2002) 

    - It is an Output Module, i.e. called by the Gate Output Manager. 
      Its function is to do a first analysis of the event preparing the data for the other output modules.
      With the help of the GateTrajectoryNavigator it reconstruct the tree of the trajectories, and 
      compute some base quantities per hit (e.g. number of compton diffusions in the phantom)
      
    - It is derived from GateVOutputModules, that provides tha base skeleton
      of all the Output classes, both for this Mgr and for the Output Modules.

    - It has a Messenger

   \sa GateVOutputModule
   \sa GateOutputModuleMessenger
   \sa GateOutputMgrMessenger


  \brief Class GateAnalysis
  \brief By Giovanni.Santin@cern.ch
  \brief $Id: GateAnalysis.hh,v 1.1.1.1 2005/02/03 13:09:53 jdbeen Exp $
*/

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

#ifndef GateAnalysis_H
#define GateAnalysis_H

#include "GateVOutputModule.hh"
#include "GateDetectorConstruction.hh"

class GateTrajectoryNavigator; 
class GateAnalysisMessenger; 

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

class GateAnalysis :  public GateVOutputModule
{
public:

  GateAnalysis(const G4String& name, GateOutputMgr* outputMgr,DigiMode digiMode);
  virtual ~GateAnalysis();

  void RecordBeginOfAcquisition();
  void RecordEndOfAcquisition();
  void RecordBeginOfRun(const G4Run * );
  void RecordEndOfRun(const G4Run * );
  void RecordBeginOfEvent(const G4Event * );

  //! Here we do the first analysis of the events
  /*! It reconstruct the tree of the trajectories with the GateTrajectoryNavigator
    and completes the hits information with global event info like the ID of the photon 
    that generated the hit, or the number of compton diffusions of this photon.
    The source ID and source position are also retrieved and stored in the hits
    \sa GateTrajectoryNavigator
  */
  void RecordEndOfEvent(const G4Event * );

  void RecordStep(const G4Step * );

  //! saves the geometry voxel information
  void RecordVoxels(GateVGeometryVoxelStore *) {};

  virtual void SetVerboseLevel(G4int val);

private:


  GateTrajectoryNavigator* m_trajectoryNavigator;

  GateAnalysisMessenger* m_analysisMessenger;

//dk
  G4int m_pos_ID;
  G4int m_gam1_ID;
  G4int m_gam2_ID;
//dk_end
            
};

#endif
